@@ -48,7 +48,8 @@ const FACTORIAL: [Decimal; 28] = [
 ];
 
 /// Trait exposing various mathematical operations that can be applied using a Decimal. This is only
-/// present when the `maths` feature has been enabled.
+/// present when the `maths` feature has been enabled, e.g. by adding the crate with
+// `cargo add rust_decimal --features maths` and importing in your Rust file with `use rust_decimal::MathematicalOps;`
 pub trait MathematicalOps {
     /// The estimated exponential function, e<sup>x</sup>. Stops calculating when it is within
     /// tolerance of roughly `0.0000002`.
@@ -180,15 +181,31 @@ impl MathematicalOps for Decimal {
             return Decimal::ONE.checked_div(exp);
         }
 
-        let mut term = *self;
+        // exp(x) = sum_i x^i / i!, let q_i := x^i/i!
+        // Avoid computing x^i directly as it will quickly outgrow exp(x) for x > 1.
+        // Instead we compute q_i = x*(x/2)*(x/3)*...*(x/i)
+
+        // First two terms are done directly: y = 1 + x
         let mut result = self.checked_add(Decimal::ONE)?;
 
-        for factorial in FACTORIAL.iter().skip(2) {
-            term = self.checked_mul(term)?;
-            let next = result + (term / factorial);
-            let diff = (next - result).abs();
-            result = next;
-            if diff <= tolerance {
+        // q_1 = x
+        let mut term = *self;
+
+        // Note: For smaller x the loop will terminate early due to the tolerance check. Only
+        // for very large x it will run more iterations, for example: exp(66.5) runs up about 186
+        // iterations.
+        const ITERATION_COUNT: u32 = 200;
+
+        for i in 2..ITERATION_COUNT {
+            // SAFETY: `i` is always a valid Decimal (and it's trivial to construct it).
+            let i_dec = Decimal::from_u32(i).unwrap();
+
+            term = self.checked_mul(term.checked_div(i_dec)?)?;
+
+            result = result.checked_add(term)?;
+
+            // Note that term is positive
+            if term <= tolerance {
                 break;
             }
         }
@@ -227,33 +244,40 @@ impl MathematicalOps for Decimal {
     }
 
     fn checked_powu(&self, exp: u64) -> Option<Decimal> {
+        if exp == 0 {
+            return Some(Decimal::ONE);
+        }
+        if self.is_zero() {
+            return Some(Decimal::ZERO);
+        }
+        if self.is_one() {
+            return Some(Decimal::ONE);
+        }
+
         match exp {
-            0 => Some(Decimal::ONE),
+            0 => unreachable!(),
             1 => Some(*self),
             2 => self.checked_mul(*self),
+            // Do the exponentiation by multiplying squares:
+            //   y = Sum (for each 1 bit in binary representation) of (2 ^ bit)
+            //   x ^ y = Sum (for each 1 bit in y) of (x ^ (2 ^ bit))
+            // See: https://en.wikipedia.org/wiki/Exponentiation_by_squaring
             _ => {
-                // Get the squared value
-                let squared = match self.checked_mul(*self) {
-                    Some(s) => s,
-                    None => return None,
-                };
-                // Square self once and make an infinite sized iterator of the square.
-                let iter = core::iter::repeat(squared);
-
-                // We then take half of the exponent to create a finite iterator and then multiply those together.
                 let mut product = Decimal::ONE;
-                for x in iter.take((exp >> 1) as usize) {
-                    match product.checked_mul(x) {
-                        Some(r) => product = r,
-                        None => return None,
-                    };
-                }
+                let mut mask = exp;
+                let mut power = *self;
 
-                // If the exponent is odd we still need to multiply once more
-                if exp & 0x1 > 0 {
-                    match self.checked_mul(product) {
-                        Some(p) => product = p,
-                        None => return None,
+                // Run through just enough 1 bits
+                for n in 0..(64 - exp.leading_zeros()) {
+                    if n > 0 {
+                        power = power.checked_mul(power)?;
+                        mask >>= 1;
+                    }
+                    if mask & 0x01 > 0 {
+                        match product.checked_mul(power) {
+                            Some(r) => product = r,
+                            None => return None,
+                        };
                     }
                 }
                 product.normalize_assign();
@@ -450,6 +474,7 @@ impl MathematicalOps for Decimal {
         if self.is_one() {
             return Some(Decimal::ZERO);
         }
+
         // This uses a very basic method for calculating log10. We know the following is true:
         //   log10(n) = ln(n) / ln(10)
         // From this we can perform some small optimizations:
@@ -457,14 +482,23 @@ impl MathematicalOps for Decimal {
         //  2. Multiplication is faster than division, so we can pre-calculate the constant 1/ln(10)
         // This allows us to then simplify log10(n) to:
         //   log10(n) = C * ln(n)
+
         // Before doing all of this however, we see if there are simple calculations to be made.
+        let scale = self.scale();
         let mut working = self.mantissa_array3();
+
+        // Check for scales less than 1 as an early exit
+        if scale > 0 && working[2] == 0 && working[1] == 0 && working[0] == 1 {
+            return Some(Decimal::from_parts(scale, 0, 0, true, 0));
+        }
+
+        // Loop for detecting bordering base 10 values
         let mut result = 0;
-        let mut invalid_early_exit = false;
+        let mut base10 = true;
         while !is_all_zero(&working) {
             let remainder = div_by_u32(&mut working, 10u32);
             if remainder != 0 {
-                invalid_early_exit = true;
+                base10 = false;
                 break;
             }
             result += 1;
@@ -472,8 +506,8 @@ impl MathematicalOps for Decimal {
                 break;
             }
         }
-        if !invalid_early_exit {
-            return Some((result - self.scale()).into());
+        if base10 {
+            return Some((result - scale as i32).into());
         }
 
         self.checked_ln().map(|result| LN10_INVERSE * result)
@@ -539,7 +573,7 @@ impl MathematicalOps for Decimal {
             // -Sin(x-π)
             return (self - Decimal::PI).checked_sin().map(|x| -x);
         }
-        if self >= &Decimal::QUARTER_PI {
+        if self > &Decimal::QUARTER_PI {
             // Cos(π2-x)
             return (Decimal::HALF_PI - self).checked_cos();
         }
@@ -585,7 +619,7 @@ impl MathematicalOps for Decimal {
             // -Cos(x-π)
             return (self - Decimal::PI).checked_cos().map(|x| -x);
         }
-        if self >= &Decimal::QUARTER_PI {
+        if self > &Decimal::QUARTER_PI {
             // Sin(π2-x)
             return (Decimal::HALF_PI - self).checked_sin();
         }
@@ -738,6 +772,9 @@ impl Pow<f64> for Decimal {
 mod test {
     use super::*;
 
+    #[cfg(not(feature = "std"))]
+    use alloc::string::ToString;
+
     #[test]
     fn test_factorials() {
         assert_eq!("1", FACTORIAL[0].to_string(), "0!");
