@@ -1,16 +1,21 @@
-use crate::constants::{POWERS_10, U32_MASK};
+use crate::constants::{MAX_SCALE_U32, POWERS_10, U32_MASK};
 
 /// Rescales the given decimal to new scale.
 /// e.g. with 1.23 and new scale 3 rescale the value to 1.230
-#[inline(always)]
+#[inline]
 pub(crate) fn rescale_internal(value: &mut [u32; 3], value_scale: &mut u32, new_scale: u32) {
+    rescale::<true>(value, value_scale, new_scale);
+}
+
+#[inline(always)]
+fn rescale<const ROUND: bool>(value: &mut [u32; 3], value_scale: &mut u32, new_scale: u32) {
     if *value_scale == new_scale {
         // Nothing to do
         return;
     }
 
     if is_all_zero(value) {
-        *value_scale = new_scale;
+        *value_scale = new_scale.min(MAX_SCALE_U32);
         return;
     }
 
@@ -32,7 +37,7 @@ pub(crate) fn rescale_internal(value: &mut [u32; 3], value_scale: &mut u32, new_
             // Any remainder is discarded if diff > 0 still (i.e. lost precision)
             remainder = div_by_u32(value, 10);
         }
-        if remainder >= 5 {
+        if ROUND && remainder >= 5 {
             for part in value.iter_mut() {
                 let digit = u64::from(*part) + 1u64;
                 remainder = if digit > U32_MASK { 1 } else { 0 };
@@ -58,6 +63,11 @@ pub(crate) fn rescale_internal(value: &mut [u32; 3], value_scale: &mut u32, new_
     }
 }
 
+#[inline]
+pub(crate) fn truncate_internal(value: &mut [u32; 3], value_scale: &mut u32, desired_scale: u32) {
+    rescale::<false>(value, value_scale, desired_scale);
+}
+
 #[cfg(feature = "legacy-ops")]
 pub(crate) fn add_by_internal(value: &mut [u32], by: &[u32]) -> u32 {
     let mut carry: u64 = 0;
@@ -236,9 +246,11 @@ pub(crate) fn div_by_u32<const N: usize>(bits: &mut [u32; N], divisor: u32) -> u
     }
 }
 
-pub(crate) fn div_by_1x(bits: &mut [u32; 3], power: usize) -> u32 {
+// This function should be used with caution. It unwraps the standard divide loop - it is intended
+// for small inputs (<10) and is optimized to be left as unchecked.
+pub(crate) fn div_by_power<const POWER: usize>(bits: &mut [u32; 3]) -> u32 {
     let mut remainder = 0u32;
-    let divisor = POWERS_10[power] as u64;
+    let divisor = POWERS_10[POWER] as u64;
     let temp = ((remainder as u64) << 32) + (bits[2] as u64);
     remainder = (temp % divisor) as u32;
     bits[2] = (temp / divisor) as u32;
@@ -266,8 +278,8 @@ pub(crate) fn shl1_internal(bits: &mut [u32], carry: u32) -> u32 {
 pub(crate) fn cmp_internal(left: &[u32; 3], right: &[u32; 3]) -> core::cmp::Ordering {
     let left_hi: u32 = left[2];
     let right_hi: u32 = right[2];
-    let left_lo: u64 = u64::from(left[1]) << 32 | u64::from(left[0]);
-    let right_lo: u64 = u64::from(right[1]) << 32 | u64::from(right[0]);
+    let left_lo: u64 = (u64::from(left[1]) << 32) | u64::from(left[0]);
+    let right_lo: u64 = (u64::from(right[1]) << 32) | u64::from(right[0]);
     if left_hi < right_hi || (left_hi <= right_hi && left_lo < right_lo) {
         core::cmp::Ordering::Less
     } else if left_hi == right_hi && left_lo == right_lo {
@@ -291,37 +303,49 @@ mod test {
     use super::*;
     use crate::prelude::*;
 
+    fn to_mantissa_array_with_scale(value: &str) -> ([u32; 3], u32) {
+        let v = Decimal::from_str(value).unwrap();
+        (v.mantissa_array3(), v.scale())
+    }
+
     #[test]
     fn it_can_rescale_internal() {
-        fn extract(value: &str) -> ([u32; 3], u32) {
-            let v = Decimal::from_str(value).unwrap();
-            (v.mantissa_array3(), v.scale())
-        }
-
         let tests = &[
-            ("1", 0, "1"),
-            ("1", 1, "1.0"),
-            ("1", 5, "1.00000"),
-            ("1", 10, "1.0000000000"),
-            ("1", 20, "1.00000000000000000000"),
-            ("0.6386554621848739495798319328", 27, "0.638655462184873949579831933"),
+            ("1", 0, "1", 0),
+            ("1", 1, "1.0", 1),
+            ("1", 5, "1.00000", 5),
+            ("1", 10, "1.0000000000", 10),
+            ("1", 20, "1.00000000000000000000", 20),
             (
-                "843.65000000",                  // Scale 8
-                25,                              // 25
-                "843.6500000000000000000000000", // 25
+                "0.6386554621848739495798319328",
+                27,
+                "0.638655462184873949579831933",
+                27,
             ),
             (
-                "843.65000000",                     // Scale 8
-                30,                                 // 30
-                "843.6500000000000000000000000000", // 28
+                "843.65000000", // Scale 8
+                25,
+                "843.6500000000000000000000000",
+                25,
             ),
+            (
+                "843.65000000", // Scale 8
+                30,
+                "843.6500000000000000000000000",
+                25, // Only fits 25
+            ),
+            ("0", 130, "0.000000000000000000000000000000", 28),
         ];
 
-        for &(value_raw, new_scale, expected_value) in tests {
-            let (expected_value, _) = extract(expected_value);
-            let (mut value, mut value_scale) = extract(value_raw);
+        for &(value_raw, new_scale, expected_value, expected_scale) in tests {
+            let (expected_value, _) = to_mantissa_array_with_scale(expected_value);
+            let (mut value, mut value_scale) = to_mantissa_array_with_scale(value_raw);
             rescale_internal(&mut value, &mut value_scale, new_scale);
             assert_eq!(value, expected_value);
+            assert_eq!(
+                value_scale, expected_scale,
+                "value: {value_raw}, requested scale: {new_scale}"
+            );
         }
     }
 
